"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const ts_simple_ast_1 = __importDefault(require("ts-simple-ast"));
const typescript_poor_man_reflection_1 = require("typescript-poor-man-reflection");
require("..");
const customExtractor_1 = require("../customExtractor");
const util_1 = require("../util");
describe('Type() and custom extractor', () => {
    it('should build custom extractor for extracting all declarations that add names in current source file global or describe(), it(), test() closures', () => {
        const project = new ts_simple_ast_1.default();
        const file = util_1.unique('test_') + '.ts';
        project.createSourceFile(file, `
describe('ff', ()=>{
interface I{
  i:number
}
it('as', ()=>{
  class C implements I {
    i:number=0
    m(){
      var foo=1
      while(true){
        var t0 = Date.now()
      }
    }
  }
})  
})
test('bar', ()=>{
  const var55 = 's'
})
const body = Type<any>()
    `);
        typescript_poor_man_reflection_1.replaceFileFunctionCall(project.getSourceFile(file), {
            extracts: {
                Type: customExtractor_1.customExtractor
            },
            extractorPrependVariableName: '__CE'
        });
        expect(project.getSourceFile(file).getText()).toContain(`const body = Type<any>({text: "any", __tsdCR_prefix: __CE[0]})`);
        expect(project.getSourceFile(file).getText()).toContain(`const __CE = ["interface I{\\n  i:number\\n}\\nclass C implements I {\\n    i:number=0\\n    m(){\\n      var foo=1\\n      while(true){\\n        var t0 = Date.now()\\n      }\\n    }\\n  }\\nconst var55 = 's'"]`);
        expect(project.getSourceFile(file).getText()).not.toContain(`\\nvar foo`);
    });
    it('should throw in case duplicate declaration names are found', () => {
        const project = new ts_simple_ast_1.default();
        const file = util_1.unique('test_') + '.ts';
        project.createSourceFile(file, `
describe('ff', ()=>{
  var b = 2
  it('as', ()=>{
    var b = 3
    const a = Type<any>()
  })  
})`);
        expect(() => typescript_poor_man_reflection_1.replaceFileFunctionCall(project.getSourceFile(file), {
            extracts: {
                Type: customExtractor_1.customExtractor
            }
        })).toThrowError();
    });
});
