"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const checkType_1 = require("../checkType");
const intermediaFunction_1 = require("./assets/intermediaFunction");
const intermediaFunction2_1 = require("./assets/intermediaFunction2");
test("should not fail if type don't match", () => {
    const result = checkType_1.checkType('number', '1');
    expect(result.pass).toBe(false);
});
test('should not fail if type match', () => {
    const result = checkType_1.checkType('number', 1);
    expect(result.pass).toBe(true);
});
test('can reference types from other files', () => {
    let result = checkType_1.checkType(value => `
      var b = Math.random()>0.5 ? 1 : '1'
      type CoolType = If<IsString<typeof b>, 'greater than 0.5', 'lower than 0.5'>
      var a: CoolType = ${value}`, 'equals 2 * 3');
    expect(result.pass).toBe(false);
    let result2 = checkType_1.checkType(value => `
      var b = Math.random()>0.5 ? 1 : '1'
      var a: If<IsString<typeof b>, 'greater than 0.5', 'lower than 0.5'> = ${value}`, 'greater than 0.5');
    expect(result2.pass).toBe(true);
});
test('should allow to pass an array value', () => {
    let result = checkType_1.checkType(value => `
      var a: KeysToTuple<Date> = ${value}`, ['these are', 'not the keys of Date']);
    expect(result.pass).toBe(false);
    result = checkType_1.checkType(value => `
      var a: KeysToTuple<Date> = ${value}`, ['toUTCString', 'toISOString', 'toJSON']);
    expect(result.pass).toBe(true);
});
test('should allow to pass an function value', () => {
    let result = checkType_1.checkType(value => `
      type F = (...args: any[])=>number
      var a: F = ${value}`, () => 'fails');
    expect(result.pass).toBe(false);
    result = checkType_1.checkType(value => `
      type F = (...args: any[])=>number
      var a: F = ${value}`, () => Math.PI);
    expect(result.pass).toBe(true);
});
test('should reference types in the global scope', () => {
    let result = checkType_1.checkType(`Global<'hello'>`, 1);
    expect(result.pass).toBe(false);
    result = checkType_1.checkType(`Global<'hello'>`, '');
    expect(result.pass).toBe(true);
});
test('should support undefined value', () => {
    const result = checkType_1.checkType(value => `
      var a: any = ${value}`, undefined);
    expect(result.pass).toBe(true);
});
test('should allow intermediate function calls', () => {
    let result = intermediaFunction_1.intermediateFunction(`number`, 1);
    expect(result.pass).toBe(true);
    result = intermediaFunction_1.intermediateFunction(`number`, []);
    expect(result.pass).toBe(false);
    result = intermediaFunction2_1.intermediateFunction2(`number`, 1);
    expect(result.pass).toBe(true);
    result = intermediaFunction2_1.intermediateFunction2(`number`, []);
    expect(result.pass).toBe(false);
});
test('expectType - the low level API', () => {
    const result = checkType_1.checkType(value => `
      var a: KeysToTuple<Date> = ${value}`, ['toUTCString', 'toISOString', 'toJSON']);
    if (!result.pass) {
        console.error(result.error, result.code, result.failErrors, result.testCode);
    }
    expect(result.pass).toBe(true);
    expect(typeof result.code).toBe('string');
    expect(result.failErrors).toHaveLength(0);
    expect(typeof result.testCode).toBe('string');
});
