"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const ts_simple_ast_1 = require("ts-simple-ast");
function Type(t) {
    return t; // we want to return undefined if that's the case should explore in user face.
}
exports.Type = Type;
const sourceFilesPrepend = {};
exports.customExtractor = (n, index, extractorPrependVariableName) => {
    if (typeof sourceFilesPrepend[n.getSourceFile().getFilePath()] === 'undefined') {
        // rootDeclarations are considered for repeated names but not added to sourceFilesPrepend
        const rootDeclarations = [];
        const declarations = n
            .getSourceFile()
            .getDescendants()
            .filter(d => {
            return (ts_simple_ast_1.TypeGuards.isCallSignatureDeclaration(d) ||
                ts_simple_ast_1.TypeGuards.isClassDeclaration(d) ||
                ts_simple_ast_1.TypeGuards.isEnumDeclaration(d) ||
                ts_simple_ast_1.TypeGuards.isVariableDeclarationList(d) ||
                ts_simple_ast_1.TypeGuards.isFunctionDeclaration(d) ||
                ts_simple_ast_1.TypeGuards.isInterfaceDeclaration(d) ||
                ts_simple_ast_1.TypeGuards.isIndexSignatureDeclaration(d) ||
                ts_simple_ast_1.TypeGuards.isTypeAliasDeclaration(d));
        })
            .filter(d => {
            const block = d.getFirstAncestorByKind(ts_simple_ast_1.SyntaxKind.Block);
            if (block &&
                block.getParent() &&
                block.getParent().getParent() &&
                ts_simple_ast_1.TypeGuards.isCallExpression(block.getParent().getParent())) {
                const ce = block.getParent().getParent();
                const id = ce.getFirstChildByKind(ts_simple_ast_1.SyntaxKind.Identifier);
                if (id) {
                    return ['describe', 'it', 'test'].includes(id.getText());
                }
            }
            else if ((d.getParent() && ts_simple_ast_1.TypeGuards.isSourceFile(d.getParent())) ||
                (d.getParent().getParent() && ts_simple_ast_1.TypeGuards.isSourceFile(d.getParent().getParent()))) {
                rootDeclarations.push(d);
            }
        });
        let sameName = '';
        let sameNameLineNumber = 0;
        declarations.some(d => {
            const dNames = getNames(d);
            return !![...declarations, ...rootDeclarations].find(d2 => d2 !== d &&
                !!getNames(d2).find(n => {
                    if (dNames.includes(n)) {
                        sameName = `"${n}" found in "${d.getText()}" and in "${d2.getText()}"`;
                        sameNameLineNumber = d.getStartLineNumber();
                        return true;
                    }
                    return false;
                }));
        });
        if (sameName) {
            throw new Error(`Declarations with same name detected !
File: "${n.getSourceFile().getFilePath()}:${sameNameLineNumber}"
Names: ${sameName}
Type() could fail, rename these identifiers in order to use it!. Aborting.`);
        }
        sourceFilesPrepend[n.getSourceFile().getFilePath()] = index;
        return {
            argument: `{text: ${JSON.stringify(`${n.getTypeArguments()[0].getText()}`)}, __tsdCR_prefix: ${extractorPrependVariableName}[${sourceFilesPrepend[n.getSourceFile().getFilePath()]}]}`,
            prependToFile: JSON.stringify(`${declarations.map(d => d.getText()).join('\n')}`)
        };
    }
    else {
        return {
            argument: `{text: ${JSON.stringify(`${n.getTypeArguments()[0].getText()}`)}, __tsdCR_prefix: ${extractorPrependVariableName}[${sourceFilesPrepend[n.getSourceFile().getFilePath()]}]}`
        };
    }
};
function getNames(n) {
    if (ts_simple_ast_1.TypeGuards.isVariableDeclarationList(n)) {
        return n.getDeclarations().map(d => d.getFirstChildByKind(ts_simple_ast_1.SyntaxKind.Identifier).getText());
    }
    else {
        return [n.getFirstChildByKind(ts_simple_ast_1.SyntaxKind.Identifier).getText()];
    }
}
