"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const ts_simple_ast_1 = require("ts-simple-ast");
const javascript_stringify_1 = require("javascript-stringify");
/** @internal */
function formatDiagnostics(d) {
    return d.map(tsd => ({
        message: ts_simple_ast_1.ts.flattenDiagnosticMessageText(tsd.compilerObject.messageText, '\n'),
        code: tsd.getCode(),
        file: tsd.getSourceFile() && tsd.getSourceFile().getFilePath(),
        length: tsd.getLength(),
        lineNumber: tsd.getLineNumber(),
        start: tsd.getStart(),
        startColumn: ts_simple_ast_1.ts.getLineAndCharacterOfPosition(tsd.getSourceFile().compilerNode, tsd.getStart()).character + 1,
        startLineNumber: ts_simple_ast_1.ts.getLineAndCharacterOfPosition(tsd.getSourceFile().compilerNode, tsd.getStart()).line + 1,
        endColumn: ts_simple_ast_1.ts.getLineAndCharacterOfPosition(tsd.getSourceFile().compilerNode, tsd.getStart() + tsd.getLength())
            .character + 1,
        endLineNumber: ts_simple_ast_1.ts.getLineAndCharacterOfPosition(tsd.getSourceFile().compilerNode, tsd.getStart() + tsd.getLength()).line + 1
    }));
}
exports.formatDiagnostics = formatDiagnostics;
let _unique = 0;
/** @internal */
function unique(prefix = '_') {
    return prefix + _unique++;
}
exports.unique = unique;
/** @internal */
function quote(s, q = '"') {
    return q + s.replace(new RegExp(q, 'g'), '\\' + q) + q;
}
exports.quote = quote;
/** @internal */
function escapeValue(v, options) {
    if (typeof v === 'string' || options.asString) {
        return quote(v + '');
    }
    else if (options.enforceJsonValues) {
        try {
            return JSON.stringify(v);
        }
        catch (error) {
            return undefined;
        }
    }
    else if (typeof v === 'undefined') {
        return 'undefined';
    }
    else if (v === null) {
        return 'null';
    }
    else {
        const s = javascript_stringify_1.stringify(v);
        if (s === undefined) {
            throw new Error('Cannot escape value ' + v);
        }
        else {
            return s;
        }
    }
}
exports.escapeValue = escapeValue;
const callsites = require('callsites');
/** @internal */
function getCallerFile() {
    const c = callsites();
    let f;
    const jestMatcher = c
        .filter(c => c.getFileName())
        .findIndex(c => c.getFileName().endsWith('dist/src/jestMatcher.js') || c.getFileName().endsWith('src/jestMatcher.ts'));
    if (jestMatcher !== -1) {
        // this is for the case of using jest matcher from an external project:
        f = c[jestMatcher + 2].getFileName();
    }
    else {
        // this works for the rest of the environments
        f = c[3] && c[3].getFileName();
    }
    // console.log('choose: ', f, 'all\n', c.map((c: any) => c.getFileName()))
    return { callerFile: f, allCallerFiles: c.map((c) => c.getFileName()).map(c => c) };
}
exports.getCallerFile = getCallerFile;
